@@ -13,6 +13,7 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+from http import client as http_client
 import json
 import os
 import time
@@ -117,6 +118,7 @@ def inspect():
 
 
 _RETRY_WAIT = 5
+_RETRY_WAIT_MAX = 30
 _RETRY_ATTEMPTS = 5
 
 
@@ -124,7 +126,6 @@ def call_inspector(data, failures):
     """Post data to inspector."""
     data['error'] = failures.get_error()
 
-    LOG.info('posting collected data to %s', CONF.inspection_callback_url)
     LOG.debug('collected data: %s',
               {k: v for k, v in data.items() if k not in _NO_LOGGING_FIELDS})
 
@@ -132,15 +133,44 @@ def call_inspector(data, failures):
     data = encoder.encode(data)
     verify, cert = utils.get_ssl_client_options(CONF)
 
+    headers = {
+        'Content-Type': 'application/json',
+        'Accept': 'application/json',
+    }
+    if CONF.global_request_id:
+        headers["X-OpenStack-Request-ID"] = CONF.global_request_id
+
+    urls = list(filter(None, CONF.inspection_callback_url.split(',')))
+
     @tenacity.retry(
         retry=tenacity.retry_if_exception_type(
-            requests.exceptions.ConnectionError),
+            (requests.exceptions.ConnectionError,
+             requests.exceptions.HTTPError)),
         stop=tenacity.stop_after_attempt(_RETRY_ATTEMPTS),
-        wait=tenacity.wait_fixed(_RETRY_WAIT),
+        wait=tenacity.wait_exponential(multiplier=1.5,
+                                       min=_RETRY_WAIT, max=_RETRY_WAIT_MAX),
         reraise=True)
     def _post_to_inspector():
-        return requests.post(CONF.inspection_callback_url, data=data,
-                             verify=verify, cert=cert)
+        for url in urls:
+            LOG.info('Posting collected data to %s', url)
+            try:
+                inspector_resp = requests.post(
+                    url, data=data, headers=headers,
+                    verify=verify, cert=cert,
+                    timeout=CONF.http_request_timeout)
+            except requests.exceptions.ConnectionError as exc:
+                if url == urls[-1]:
+                    raise
+                LOG.warning("Connection error when accessing %s, trying the "
+                            "next URL. Error: %s", url, exc)
+            else:
+                break
+
+        if (inspector_resp.status_code >= 500
+                or inspector_resp.status_code == http_client.CONFLICT):
+            raise requests.exceptions.HTTPError(response=inspector_resp)
+
+        return inspector_resp
 
     resp = _post_to_inspector()
     if resp.status_code >= 400:
@@ -355,6 +385,16 @@ def collect_pci_devices_info(data, failures):
                 LOG.warning('Wrong format of PCI revision in PCI '
                             'device %s: %s', subdir, exc)
 
+        pci_numa_node_id = None
+        pci_numa_path = os.path.join(pci_devices_path, subdir, 'numa_node')
+        if os.path.isfile(pci_numa_path):
+            try:
+                with open(pci_numa_path) as vendor_numa_node:
+                    pci_numa_node_id = vendor_numa_node.read().strip()
+            except IOError as exc:
+                LOG.warning('Failed to gather numa_node id '
+                            'from PCI device %s: %s', subdir, exc)
+
         LOG.debug(
             'Found a PCI device with vendor id %s, product id %s, class %s '
             'and revision %s', vendor, device, pci_class, pci_revision)
@@ -362,7 +402,9 @@ def collect_pci_devices_info(data, failures):
                                  'product_id': device,
                                  'class': pci_class,
                                  'revision': pci_revision,
-                                 'bus': subdir})
+                                 'bus': subdir,
+                                 'numa_node_id': pci_numa_node_id})
+
     data['pci_devices'] = pci_devices_info
 
 
